#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <assert.h>
#include "inout.h"
#include "invariant.h"
#include "graphe.h"
#include "connexe.h"
#include "hamiltonien.h"

#define oMAX 10
#define aMAX ( (( oMAX-1)*oMAX) / 2 )

typedef unsigned long long ullong;

typedef ullong picographe;

#define ONE  ( ( ullong ) 1 )

#define BIT(k)  ( ONE << (k) )


typedef int permutation[oMAX];

int X[aMAX], Y[aMAX], Z[oMAX][oMAX];
ullong TRANS[aMAX];
ullong SHIFT[aMAX];

int Ordre, Arete;

ullong limite;
int *table;
int taille;
int classe;
extern int verbe;


graphe picotographe(picographe w)
{
    graphe res = initGraphe(Ordre);
    int i, j;
    for (i = 0; i < Ordre; i++)
	for (j = i + 1; j < Ordre; j++)
	    if (BIT(Z[i][j]) & w)
		res.mat[i][j] = res.mat[j][i] = 1;
    return res;
}

void initpicographe(int n)
{
    int i, j, k = 0;
    for (i = 0; i < n; i++)
	for (j = i + 1; j < n; j++) {
	    X[k] = i;
	    Y[k] = j;
	    Z[i][j] = k;
	    Z[j][i] = k;
	    k++;
	}
    Ordre = n;
    Arete = n * (n - 1) / 2;
    limite = 1;
    limite <<= Arete;
    table = calloc(limite, sizeof(int));
    for (k = 0; k < Arete; k++) {
	i = X[k];
	j = Y[k];
	if (i < 2)
	    i = 1 - i;
	if (j < 2)
	    i = 1 - j;
	TRANS[k] = BIT(Z[i][j]);
    }
    for (k = 0; k < Arete; k++) {
	i = X[k];
	j = Y[k];
	i = X[k];
	i = (i + 1) % Ordre;
	j = Y[k];
	j = (j + 1) % Ordre;
	SHIFT[k] = BIT(Z[i][j]);
    }
}

picographe shift(picographe w)
{
    int k;
    picographe res = 0;
    for (k = 0; k < Arete; k++)
	if (BIT(k) & w) {
	    res ^= SHIFT[k];
	}
    return res;
}

picographe trans(picographe w)
{
    int k;
    picographe res = 0;
    for (k = 0; k < Arete; k++)
	if (BIT(k) & w) {
	    res ^= TRANS[k];
	}
    return res;
}

void PPR(picographe w)
{
    picographe s;
    table[w] = classe;
    taille++;
    s = shift(w);
    if (!table[s])
	PPR(s);

    s = trans(w);
    if (!table[s])
	PPR(s);
}

int *classification(int *nbcls)
{
    picographe w;
    classe = 0;
    for (w = 0; w < limite; w++)
	if (!table[w]) {
	    classe++;
	    taille = 0;
	    PPR(w);
	}
    *nbcls = classe;
    return table;
}
